@@ -37,8 +37,12 @@
 void a_datasource_gc_init();
 #endif
 
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 
 #include <glib/gprintf.h>
 #include <glib/gi18n.h>
@@ -206,5 +210,7 @@ int main( int argc, char *argv[] )
   a_dems_uninit ();
   a_preferences_uninit ();
 
+  curl_download_uninit();
+
   return 0;
 }
