@@ -45,6 +45,7 @@ static const char *menu_xml =
 #ifdef VIK_CONFIG_EXPEDIA
 	"      <menuitem action='ModeExpedia'/>"
 #endif
+	"      <menuitem action='ModeLatLon'/>"
 	"      <menuitem action='ModeMercator'/>"
 	"      <separator/>"
 	"      <menuitem action='GotoDefaultLocation'/>"
@@ -52,11 +53,18 @@ static const char *menu_xml =
 	"      <menuitem action='GotoLL'/>"
 	"      <menuitem action='GotoUTM'/>"
 	"      <separator/>"
-	"      <menuitem action='ShowScale'/>"
-	"      <menuitem action='ShowCenterMark'/>"
 	"      <menuitem action='SetBGColor'/>"
+	"      <menuitem action='SetHLColor'/>"
 	"      <menuitem action='FullScreen'/>"
-	"      <menuitem action='ViewSidePanel'/>"
+	"      <menu action='SetShow'>"
+	"          <menuitem action='ShowScale'/>"
+	"          <menuitem action='ShowCenterMark'/>"
+	"          <menuitem action='ShowHighlight'/>"
+	"          <menuitem action='ViewSidePanel'/>"
+	"          <menuitem action='ViewStatusBar'/>"
+	"          <menuitem action='ViewToolbar'/>"
+	"          <menuitem action='ViewMainMenu'/>"
+	"      </menu>"
 	"      <separator/>"
 	"      <menuitem action='ZoomIn'/>"
 	"      <menuitem action='ZoomOut'/>"
@@ -99,6 +107,7 @@ static const char *menu_xml =
 	"      <menuitem action='Pan'/>"
 	"      <menuitem action='Zoom'/>"
 	"      <menuitem action='Ruler'/>"
+	"      <menuitem action='Select'/>"
 	"    </menu>"
 	"    <menu action='Help'>"
 	"      <menuitem action='HelpEntry'/>"
@@ -130,6 +139,7 @@ static const char *menu_xml =
 	"    <toolitem action='Pan'/>"
 	"    <toolitem action='Zoom'/>"
 	"    <toolitem action='Ruler'/>"
+	"    <toolitem action='Select'/>"
 	"    </placeholder>"
 	"  </toolbar>"
 	"</ui>"
